<?php

class Mage_Core_Helper_Security
{

    private $invalidBlockActions
        = [
            ['block' => Mage_Page_Block_Html_Topmenu_Renderer::class, 'method' => 'render'],
            ['block' => Mage_Core_Block_Template::class, 'method' => 'fetchView'],
        ];

    /**
     * @param Mage_Core_Block_Abstract $block
     * @param string                   $method
     * @param string[]                 $args
     *
     * @throws Mage_Core_Exception
     */
    public function validateAgainstBlockMethodBlacklist(Mage_Core_Block_Abstract $block, $method, array $args)
    {
        foreach ($this->invalidBlockActions as $action) {
            $calledMethod = strtolower($method);
            if (str_contains($calledMethod, '::')) {
                $calledMethod = explode('::', $calledMethod)[1];
            }
            if (($block instanceof $action['block'] && strtolower($action['method']) === $calledMethod)) {
                Mage::throwException(
                    sprintf('Action with combination block %s and method %s is forbidden.', get_class($block), $method)
                );
            }
        }
    }

    public function validateIsSubdirectory($parentDirectory, $subDirectory)
    {
        $parentDirectory = (array)$parentDirectory;
        foreach ($parentDirectory as $parent) {
            if (str_starts_with(
                $this->realpathNonExistingPath($subDirectory),
                $this->realpathNonExistingPath($parent)
            )) {
                return $parentDirectory;
            }
        }
        Mage::throwException(
            sprintf(
                'Path "%s" is outside of "%s", this should not happen.',
                $subDirectory,
                implode(', ', $parentDirectory)
            )
        );
    }

    private function realpathNonExistingPath($path)
    {
        $directories = array_filter(explode(DS, $path));
        $removedDirectories = [];
        // Walk the directory upwards... (see while())
        do {
            // until you find an existing directory and realpath it
            $existingRealPath = realpath(DS . implode(DS, $directories));
            if (!$existingRealPath) {
                continue;
            }
            // then add the nonexisting path back to it
            return $existingRealPath . DS . implode(DS, array_reverse($removedDirectories));
        } while ($removedDirectories[] = array_pop($directories));
    }
}
